{Procedury graficzne dla trybu 13H do Turbo Pascala}
{autor: Sebastian Pawlak, Warszawa 1996}

UNIT GRAPH13H;
INTERFACE  {obszar deklaracji zmiennych, procedur i funkcji}
           {widocznych dla uzytkownika}
{Zmienne widoczne dla uzytkownika}
VAR SegVga,OfsVga:word;

{Procedura przelaczajaca tryby graficzne}
PROCEDURE InitGraph (Tryb:byte);

{Procedura stawiajaca punkt o wspolrzednych X, Y i danym}
{kolorze}
PROCEDURE PutPixel (X,Y:word;Kolor:byte);

{Procedura kopiujaca wycinek obrazu do bufora}
{X1, Y1 - lewy gorny rog  }
{X2, Y2 - prawy dolny rog}
{SegBuf, OfsBuf - segment i offset bufora}
PROCEDURE GetBitMap (X1,Y1,X2,Y2,SegBuf,OfsBuf:word);

{Procedura wyswietlajaca zawartosc bufora na ekranie}
{X1, Y1 - lewy gorny rog  }
{SegBuf, OfsBuf - segment i offset bufora}
PROCEDURE PutBitMap (X1,Y1,SegBuf,OfsBuf:word);

{Procedura wyswietlajaca sprite`a}
{X1, Y1 - lewy gorny rog   }
{SegBuf, OfsBuf - segment i offset bufora}
{KolorTla - kolor tla bitmapy, ktore nie bedzie wyswietlane}
PROCEDURE PutSprite (X1,Y1,SegBuf,OfsBuf:word;KOLORTLA:BYTE);

{Procedura ustawiajaca parametry RGB dla bloku kolorow}
PROCEDURE SetRGBBlock (OdKoloru,IloscKolorow,SegBuf,OfsBuf:word);

{Procedura czekajaca na powrot pionowy}
PROCEDURE GraphWait;

{Procedura kopiujaca dane z jednego miejsca w pamieci}
{w inne}
PROCEDURE CopyMem (SegSource,OfsSource,SegBuf,OfsBuf,Size:word);

{Format graficzny UBM "Uproszczona BitMapa"}
{Procedura wczytujaca obraz i palete kolorow do pamieci}
PROCEDURE LoadUBM (Name:string;SegBuf,OfsBuf,SegPalette,OfsPalette:word);

{Funkcja odczytujaca szerokosc obrazu zapisanego w pliku}
{Format graficzny UBM}
FUNCTION GetUBMWidth (Name:string):word;

{Funkcja odczytujaca wysokosc obrazu zapisanego w pliku}
{Format graficzny UBM}
FUNCTION GetUBMHeight (Name:string):word;

{Funkcja odczytujaca, od ktorego koloru zaczyna sie definicja}
{palety kolorow}
FUNCTION GetFirstColor (Name:string):byte;

{Funkcja odczytujaca, ilosc elementow w definicji palety}
{kolorow}
FUNCTION GetLengthPalette (Name:string):word;

IMPLEMENTATION  {obszar definicji zmiennych, procedur}

{Procedura przelaczajaca tryby graficzne}
PROCEDURE InitGraph (Tryb:byte); ASSEMBLER;
ASM
 MOV AH,00H
 MOV AL,Tryb
 INT 10H
END;

{Procedura stawiajaca punkt o wspolrzednych X, Y i danym}
{kolorze}
PROCEDURE PutPixel (X,Y:word;Kolor:byte); ASSEMBLER;
ASM
 PUSH ES
 MOV AX,SegVga
 MOV ES,AX
 MOV AX,X
 MOV BX,Y
 MOV CL,6
 SHL BX,CL      {mnozy wspl. Y przez 2 do potegi 6, czyli 64}
 MOV CX,BX      {wynik mnozenia zapisuje w CX}
 SHL BX,1       {BX mnozy przez 2}
 SHL BX,1       {i jeszcze przez 2 ; razem 4}
 ADD BX,CX      {sumuje 64*4 i 64 co daje 320 (razy Y)}
 ADD BX,AX      {do 320*Y dodaje X}
 ADD BX,OfsVga
 MOV DL,Kolor
 MOV ES:[BX],DL
 POP ES
END;

{Procedura kopiujaca wycinek obrazu do bufora}
{X1, Y1 - lewy gorny rog  }
{X2, Y2 - prawy dolny rog}
{SegBuf, OfsBuf - segment i offset bufora}
PROCEDURE GetBitMap (X1,Y1,X2,Y2,SegBuf,OfsBuf:word); ASSEMBLER;
ASM
 PUSH DS
 PUSH ES

 MOV BX,Y1    {Wyliczenie offsetu lewego}
 MOV CL,6     {gornego rogu}
 SHL BX,CL
 MOV AX,BX
 SHL BX,1
 SHL BX,1
 ADD AX,BX
 ADD AX,X1
 MOV SI,AX      {DS:SI = pamiec zrodla}
 ADD SI,OfsVga
 MOV AX,SegVga
 MOV DS,AX

 MOV AX,SegBuf  {ES:DI = pamiec bufora}
 MOV ES,AX
 MOV DI,OfsBuf

 MOV BX,X2      {Wyliczenie szerokosci obiektu}
 SUB BX,X1
 MOV ES:[DI],BX {Zapis szerokosci do 1-2 bajtu bufora}
 INC DI
 INC DI
 INC BX

 MOV AX,BX      {Dzieli szrokosc przez dwa dla}
 MOV DL,2       {2-bajtowego kopiowania do bufora}
 DIV DL
 CMP AH,0
 JE @BEZ_RESZTY
 MOV AH,1
@BEZ_RESZTY:

 MOV CX,Y2      {Wyliczenie wysokosci obiektu}
 SUB CX,Y1
 MOV ES:[DI],CX {Zapis wysokosci do 3-4 bajtu bufora}
 INC DI
 INC DI
 INC CX

@NEXT_LINE:
 PUSH CX
 MOV CH,0
 MOV CL,AL
 REP MOVSW   {Kopiowanie danych 2-bajtowo do bufora}
 MOV CL,AH
 REP MOVSB   {Kopiowanie bajtu dla szerokosci nieparzystej}
 SUB SI,BX    {Ustawienie offsetu na poczatku}
 ADD SI,320   {nastepnego wiersza}
 POP CX
 LOOP @NEXT_LINE {Pobiera nastepna linie}

 POP ES
 POP DS
END;

{Procedura wyswietlajaca zawartosc bufora na ekranie}
{X1, Y1 - lewy gorny rog  }
{SegBuf, OfsBuf - segment i offset bufora}
PROCEDURE PutBitMap (X1,Y1,SegBuf,OfsBuf:word); ASSEMBLER;
ASM
 PUSH DS
 PUSH ES

 MOV BX,Y1    {Wyliczenie offsetu lewego gornego rogu}
 MOV CL,6
 SHL BX,CL
 MOV AX,BX
 SHL BX,1
 SHL BX,1
 ADD AX,BX
 ADD AX,X1

 MOV DI,AX       {ES:DI = pamiec miejsca docelowego}
 ADD DI,OfsVga
 MOV AX,SegVga
 MOV ES,AX

 MOV AX,SegBuf  {DS:SI = pamiec zrodla}
 MOV DS,AX
 MOV SI,OfsBuf

 MOV BX,DS:[SI] {Odczyt szerokosci obiektu}
 INC SI
 INC SI
 INC BX

 MOV AX,BX    {Dzieli szerokosc przez dwa dla}
 MOV DL,2       {2-bajtowego kopiowania na ekran}
 DIV DL
 CMP AH,0
 JE @BEZ_RESZTY
 MOV AH,1
@BEZ_RESZTY:

 MOV CX,DS:[SI] {Odczyt wysokosci obiektu}
 INC SI
 INC SI
 INC CX

@NEXT_LINE:
 PUSH CX
 MOV CH,0
 MOV CL,AL
 REP MOVSW       {Kopiowanie danych 2-bajtowo}
 MOV CL,AH
 REP MOVSB   {Kopiowanie bajtu dla szerokosci nieparzystej}
 SUB DI,BX    {Ustawienie offsetu na poczatku}
 ADD DI,320   {nastepnego wiersza}
 POP CX
 LOOP @NEXT_LINE {Wyswietla nastepna linie}

 POP ES
 POP DS
END;

{Procedura wyswietlajaca sprite`a}
{X1, Y1 - lewy gorny rog   }
{SegBuf, OfsBuf - segment i offset bufora}
{KolorTla - kolor tla bitmapy, ktore nie bedzie wyswietlane}
PROCEDURE PutSprite (X1,Y1,SegBuf,OfsBuf:word;KOLORTLA:BYTE); ASSEMBLER;
ASM
 PUSH DS
 PUSH ES

 MOV BX,Y1    {Wyliczenie offsetu lewego gornego rogu}
 MOV CL,6
 SHL BX,CL
 MOV AX,BX
 SHL BX,1
 SHL BX,1
 ADD AX,BX
 ADD AX,X1

 MOV DI,AX       {ES:DI = pamiec celu (obrazu)}
 ADD DI,OfsVga
 MOV AX,SegVga
 MOV ES,AX

 MOV AX,SegBuf  {DS:SI = pamiec bufora}
 MOV DS,AX
 MOV SI,OfsBuf

 MOV BX,DS:[SI] {Odczyt szerokosci obiektu}
 INC SI
 INC SI
 INC BX

 MOV CX,DS:[SI] {Odczyt wysokosci obiektu}
 INC SI
 INC SI
 INC CX

 MOV AH,KolorTla

@NEXT_LINE:
 PUSH CX
 MOV CX,BX
@NEXT_PIXEL:
 MOV AL,DS:[SI]
 CMP AL,AH
 JE @OMIN_WYSWIETLANIE
 MOV ES:[DI],AL
@OMIN_WYSWIETLANIE:
 INC SI
 INC DI
 LOOP @NEXT_PIXEL
 SUB DI,BX    {Ustawienie offsetu na poczatku}
 ADD DI,320   {nastepnego wiersza}
 POP CX
 LOOP @NEXT_LINE {Nastepna linia}

 POP ES
 POP DS
END;

{Procedura ustawiajaca parametry RGB dla bloku kolorow}
PROCEDURE SetRGBBlock (OdKoloru,IloscKolorow,SegBuf,OfsBuf:word); ASSEMBLER;
ASM
 PUSH DS

 {do rejestru 3c8H zostanie wpisany numer koloru,}
 {od ktorego zaczyna sie definicja palety kolorow}
 MOV DX,03c8H
 MOV AX,OdKoloru
 OUT DX,AL

 MOV CX,IloscKolorow
 MOV DX,03c9H  {do tego rejestru beda wpisywane R, G i B}

 {adres tablicy z definicja palety}
 MOV AX,SegBuf
 MOV DS,AX
 MOV SI,OfsBuf

@NASTEPNY_RGB:
 LODSW     {pobranie z tablicy 2 bajtow (R i G)}
 OUT DX,AL {wpisanie do rejestru skladowej R}
 MOV AL,AH
 OUT DX,AL {wpisanie skladowej G}
 LODSB     {pobranie z tablicy 3 bajtu (Blue)}
 OUT DX,AL {wpisanie do rejetru skladowej B (Blue)}
 LOOP @NASTEPNY_RGB {operacja zostaje powtorzona}
                    {dla nastepnego koloru}
 POP DS
END;

{Procedura czekajaca na powrot pionowy}
PROCEDURE GraphWait; ASSEMBLER;
ASM
 MOV DX,03daH
@CZEKAJ:
 IN AL,DX
 TEST AL,8
 JE @CZEKAJ
END;

{Procedura kopiujaca dane z jednego miejsca w pamieci}
{w inne}
PROCEDURE CopyMem (SegSource,OfsSource,SegBuf,OfsBuf,Size:word); ASSEMBLER;
ASM
 PUSH DS
 PUSH ES

 MOV BX,Size       {ilosc bajtow do skopiowania}

 MOV AX,SegBuf     {ES:DI = cel}
 MOV ES,AX
 MOV DI,OfsBuf

 MOV SI,OfsSource
 MOV AX,SegSource  {DS:SI = zrodlo}
 MOV DS,AX

 MOV DX,0
 MOV AX,BX        {Dzielenie ilosci bajtow przez 2 dla}
 MOV BX,2         {2-bajtowego kopiowania}
 DIV BX
 CMP DL,0
 JE @BEZ_RESZTY
 MOV DL,1
@BEZ_RESZTY:

 MOV CX,AX
 REP MOVSW        {kopiowanie slowami}
 MOV CL,DL
 REP MOVSB        {kopiowanie bajtu dla nieparzystej ilosci}

 POP ES
 POP DS
END;

VAR NameFile:string;
    HandleFile:word;

{Funkcja otwierajaca plik i zwracajaca "uchwyt"}
FUNCTION OpenFile (Name:string):word;
BEGIN
 NameFile:=Name;
 ASM
  MOV DX,OFFSET NameFile
  INC DX
  MOV AH,3dH
  MOV AL,2
  INT 21H
  JAE @PLIK_OTWARTY
  MOV HandleFile,0
  JMP @KONIEC
 @PLIK_OTWARTY:
  MOV HandleFile,AX
 @KONIEC:
 END;
 OpenFile:=HandleFile;
END;

{Procedura przesuwajaca wskaznik w pliku na podany}
PROCEDURE SeekFile (Handle,Ofs:word); ASSEMBLER;
ASM
 MOV AH,42H
 MOV AL,0
 MOV CX,0
 MOV DX,Ofs
 MOV BX,Handle
 INT 21H
END;

{Procedura czytajaca z pliku blok danych}
PROCEDURE ReadFile (Handle,SegBuf,OfsBuf,Size:word); ASSEMBLER;
ASM
 PUSH DS
 MOV AX,SegBuf
 MOV DS,AX
 MOV DX,OfsBuf
 MOV AH,3fH
 MOV BX,Handle
 MOV CX,Size
 INT 21H
 POP DS
END;

{Procedura zamykajaca plik}
PROCEDURE CloseFile (Handle:word); ASSEMBLER;
ASM
 MOV BX,Handle
 MOV AH,3eH
 INT 21H
END;

{Format graficzny UBM "Uproszczona BitMapa"}
{Procedura wczytujaca obraz i palete kolorow do pamieci}
PROCEDURE LoadUBM (Name:string;SegBuf,OfsBuf,SegPalette,OfsPalette:word);
VAR UBMWidth:word;
    UBMHeight:word;
    Handle:word;
    OdKoloru:byte;
    IloscKolorow:byte;

BEGIN
 Handle:=OpenFile (Name);

 {ten format graficzny umozliwia zapisanie tylko wycinka}
 {palety - tu odczytywane jest, od ktorego koloru zaczyna}
 {sie paleta i ile elementow zawiera}
 ReadFile (Handle,Seg (OdKoloru),Ofs (OdKoloru),1);
 ReadFile (Handle,Seg (IloscKolorow),Ofs (IloscKolorow),1);

 {zeby informacja o ilosci kolorow zmiescila sie w jednym}
 {bajcie jej wartosc jest zmniejszana o 1, stad zwiekszanie}
 {zmiennej IloscKolorow w nastepnym wierszu}
 ReadFile (Handle,SegPalette,OfsPalette,(IloscKolorow+1)*3);

 ReadFile (Handle,Seg (UBMWidth),Ofs (UBMWidth),2);
 ReadFile (Handle,Seg (UBMHeight),Ofs (UBMHeight),2);

 SeekFile (Handle,(IloscKolorow+1)*3+2);
 ReadFile (Handle,SegBuf,OfsBuf,(UBMWidth+1)*(UBMHeight+1)+4);

 CloseFile (Handle);
END;

{Funkcja odczytujaca szerokosc obrazu zapisanego w pliku}
{Format graficzny UBM}
FUNCTION GetUBMWidth (Name:string):word;
VAR UbmWidth:word;
    Handle:word;
    IloscKolorow:byte;

BEGIN
 Handle:=OpenFile (Name);
 SeekFile (Handle,1);
 ReadFile (Handle,Seg (IloscKolorow),Ofs (IloscKolorow),1);
 SeekFile (Handle,(IloscKolorow+1)*3+2);
 ReadFile (Handle,Seg (UbmWidth),Ofs (UbmWidth),2);
 CloseFile (Handle);
 GetUBMWidth:=UbmWidth+1;
END;

{Funkcja odczytujaca wysokosc obrazu zapisanego w pliku}
{Format graficzny UBM}
FUNCTION GetUBMHeight (Name:string):word;
VAR UbmHeight:word;
    Handle:word;
    IloscKolorow:byte;

BEGIN
 Handle:=OpenFile (Name);
 SeekFile (Handle,1);
 ReadFile (Handle,Seg (IloscKolorow),Ofs (IloscKolorow),1);
 SeekFile (Handle,(IloscKolorow+1)*3+4);
 ReadFile (Handle,Seg (UbmHeight),Ofs (UbmHeight),2);
 CloseFile (Handle);
 GetUBMHeight:=UbmHeight+1;
END;

{Funkcja odczytujaca, od ktorego koloru zaczyna sie definicja}
{palety kolorow}
FUNCTION GetFirstColor (Name:string):byte;
VAR FirstColor:byte;
    Handle:word;

BEGIN
 Handle:=OpenFile (Name);
 ReadFile (Handle,Seg (FirstColor),Ofs (FirstColor),1);
 CloseFile  (Handle);
 GetFirstColor:=FirstColor;
END;

{Funkcja odczytujaca, ilosc elementow w definicji palety}
{kolorow}
FUNCTION GetLengthPalette (Name:string):word;
VAR LengthPalette:byte;
    Handle:word;

BEGIN
 Handle:=OpenFile (Name);
 SeekFile (Handle,1);
 ReadFile (Handle,Seg (LengthPalette),Ofs (LengthPalette),1);
 CloseFile (Handle);
 GetLengthPalette:=LengthPalette+1;
END;

BEGIN
 SegVga:=$0a000;
 OfsVga:=0;
END.